import re

BIZ_REVIEW_URL = "https://www.yelp.com/biz/{}?hrid={}"


def get_biz_review_url(biz_id, review_id):
    return BIZ_REVIEW_URL.format(biz_id, review_id)


def review_id_on_page(page_text, review_id):
    return bool(re.search(review_id, page_text))



from tests.scraper.resources import get_test_resource_path
from yelp_fetcher.scraper.biz import get_biz_review_url, review_id_on_page


def test_get_biz_review_url():
    # Given
    expected_urls = {
        ("biz0", "rev0"): "https://www.yelp.com/biz/biz0?hrid=rev0",
        ("biz1", "rev1"): "https://www.yelp.com/biz/biz1?hrid=rev1",
        ("biz2", "rev2"): "https://www.yelp.com/biz/biz2?hrid=rev2",
    }

    for (biz_id, review_id), expected_url in expected_urls.items():
        # When
        url = get_biz_review_url(biz_id, review_id)

        # Then
        assert url == expected_url


def test_review_id_on_page_yes():
    # Given
    filenames = {
        "ct_w8SKNRmrwxqmXTRe_rw": "biz/chipotle-mexican-grill-cerritos-9?hrid=ct_w8SKNRmrwxqmXTRe_rw.html",
        "HcLsMKuQT3KHgf6-0J0zMw": "biz/aloha-family-billiards-buena-park?hrid=HcLsMKuQT3KHgf6-0J0zMw.html",
        "VcmUwOpWndm0AKvvmwW9NA": "biz/bunker21-artesia?hrid=VcmUwOpWndm0AKvvmwW9NA.html",
    }

    # When
    for review_id, path in filenames.items():
        with open(get_test_resource_path(path)) as f:
            result = review_id_on_page(f.read(), review_id)

            # Then
            assert result


def test_review_id_on_page_no():
    # Given
    filenames = {
        "ofWhnV6m26kMyCpUjT56XA": "biz/chipotle-mexican-grill-cerritos-9?hrid=ofWhnV6m26kMyCpUjT56XA.html",
        "XMjUx1JAw4GVEhGGY4MoYw": "biz/aloha-family-billiards-buena-park?hrid=XMjUx1JAw4GVEhGGY4MoYw.html",
        "4Kw3pCYa_cm3g_lyqppC9g": "biz/bunker21-artesia?hrid=4Kw3pCYa_cm3g_lyqppC9g.html",
    }

    # When
    for review_id, path in filenames.items():
        with open(get_test_resource_path(path)) as f:
            result = review_id_on_page(f.read(), review_id)

            # Then
            assert not result



import requests
from yelp_fetcher.exceptions import FetchError, YelpRobotError
from yelp_fetcher.scraper.bs4_util import to_soup

ROBOT_ERROR_MESSAGE = (
    "Hey there! Before you continue, we just need to check that you're not a robot."
)


def fetch_html(url):
    response = requests.get(url)
    text = response.text
    if response.status_code != 200:
        raise FetchError(f"Fetch error for: {url}")
    elif ROBOT_ERROR_MESSAGE in text:
        raise YelpRobotError()
    return text


def fetch_soup(url):
    html = fetch_html(url)
    return to_soup(html)



from tests.scraper.resources import get_test_resource_soup
from yelp_fetcher.scraper.user_details import (
    get_user_city,
    get_user_details_url,
    get_user_name,
    get_user_review_count,
)


def test_get_user_details_url():
    # Given
    expected_urls = {
        "user_id_0": "https://www.yelp.com/user_details?userid=user_id_0",
        "user_id_1": "https://www.yelp.com/user_details?userid=user_id_1",
        "user_id_2": "https://www.yelp.com/user_details?userid=user_id_2",
    }

    for user_id, expected_url in expected_urls.items():
        # When
        url = get_user_details_url(user_id)

        # Then
        assert url == expected_url


def test_get_user_name():
    # Given
    test_cases = {
        get_test_resource_soup(
            "user_details/userid=5prk8CtPPBHNpa6BOja2ug.html"
        ): "Sam K.",
        get_test_resource_soup(
            "user_details/userid=uti2qXa5mupl2YqtrxHaOg.html"
        ): "G K.",
        get_test_resource_soup(
            "user_details/userid=Gx9m-uVNSAua8KdVyiQfGA.html"
        ): "Christina L.",
    }

    for user_page, expected in test_cases.items():
        # When
        result = get_user_name(user_page)

    # Then
    assert result == expected


def test_get_user_city():
    # Given
    test_cases = {
        get_test_resource_soup(
            "user_details/userid=5prk8CtPPBHNpa6BOja2ug.html"
        ): "Irvine, CA",
        get_test_resource_soup(
            "user_details/userid=uti2qXa5mupl2YqtrxHaOg.html"
        ): "Arcadia, CA",
        get_test_resource_soup(
            "user_details/userid=Gx9m-uVNSAua8KdVyiQfGA.html"
        ): "San Francisco, CA",
    }

    for user_page, expected in test_cases.items():
        # When
        result = get_user_city(user_page)

    # Then
    assert result == expected


def test_get_user_review_count():
    # Given
    test_cases = {
        get_test_resource_soup("user_details/userid=5prk8CtPPBHNpa6BOja2ug.html"): "17",
        get_test_resource_soup("user_details/userid=uti2qXa5mupl2YqtrxHaOg.html"): "14",
        get_test_resource_soup(
            "user_details/userid=Gx9m-uVNSAua8KdVyiQfGA.html"
        ): "223",
    }

    for user_page, expected in test_cases.items():
        # When
        result = get_user_review_count(user_page)

    # Then
    assert result == expected



from bs4 import BeautifulSoup


def to_soup(text):
    return BeautifulSoup(text, "html.parser")


def get_elements_by_classname(soup: BeautifulSoup, classname):
    """Use this method when there are anywhere from 0 to N elements with the class."""
    return soup.find_all(class_=classname)


def get_element_by_classname(soup: BeautifulSoup, classname):
    """Use this method when there is exactly 1 element with the class."""
    results = soup.find_all(class_=classname)
    if len(results) == 0:
        return None
    return results[0]



from tests.scraper.resources import get_test_resource_soup
from yelp_fetcher.scraper.user_details_reviews_self import (
    ScrapedReview,
    get_user_biz_reviews,
    get_user_details_reviews_self_url,
)


def test_get_user_details_reviews_self_url():
    # Given
    user_id = "user_id"
    expected_urls = {
        0: "https://www.yelp.com/user_details_reviews_self?rec_pagestart=0&userid=user_id",
        1: "https://www.yelp.com/user_details_reviews_self?rec_pagestart=10&userid=user_id",
        2: "https://www.yelp.com/user_details_reviews_self?rec_pagestart=20&userid=user_id",
    }

    for page, expected_url in expected_urls.items():
        # When
        url = get_user_details_reviews_self_url(user_id, page)

        # Then
        assert url == expected_url


def test_get_user_biz_reviews():
    # Given
    review_page_0 = get_test_resource_soup(
        "user_details_reviews_self/user_details_reviews_self?userid=5prk8CtPPBHNpa6BOja2ug.html"
    )

    # When
    result = get_user_biz_reviews(review_page_0)

    # Then
    expected = [
        ScrapedReview(
            "bunker21-artesia", "Bunker21", "kzJihaxgEIphSySgBNNOiA", "1/2/2020",
        ),
        ScrapedReview(
            "chick-fil-a-cerritos",
            "Chick-fil-A",
            "dy_dxor2rVhpNuSUrrBw0w",
            "12/30/2019",
        ),
        ScrapedReview(
            "chipotle-mexican-grill-cerritos-9",
            "Chipotle Mexican Grill",
            "ofWhnV6m26kMyCpUjT56XA",
            "1/4/2020",
        ),
        ScrapedReview(
            "han-yang-buena-park", "Han Yang", "A_mJavYl2b4_oitezbpR8w", "12/28/2019",
        ),
        ScrapedReview(
            "la-mirada-golf-course-la-mirada",
            "La Mirada Golf Course",
            "7lJZPiK5-rBkojTxYS6R2w",
            "1/1/2020",
        ),
        ScrapedReview(
            "myung-dong-kyoja-anaheim-5",
            "Myung Dong Kyoja",
            "X0_-zEk0_CKj24wEEoau9A",
            "1/1/2020",
        ),
        ScrapedReview(
            "panda-express-cerritos",
            "Panda Express",
            "-eb-SRIXvQ_tyJKFldJMyw",
            "12/29/2019",
        ),
        ScrapedReview(
            "roger-dunn-golf-shops-seal-beach-3",
            "Roger Dunn Golf Shops",
            "Ve3BLwnfMTXZMwCViW-MLg",
            "12/31/2019",
        ),
        ScrapedReview(
            "thai-addict-cuisine-buena-park-2",
            "Thai Addict Cuisine",
            "ZfHJEeZFKTPFwAJGMYiXrA",
            "12/31/2019",
        ),
        ScrapedReview("yoko-buena-park", "Yoko", "MS1f3LaFfHeh5kcF2qt-1A", "1/3/2020",),
    ]
    assert set(result) == set(expected)


def test_get_user_biz_reviews_none():
    # Given
    page_no_reviews = get_test_resource_soup(
        "user_details_reviews_self/user_details_reviews_self?userid=yOdiJSKdFuE6EsAfd7jDKQ.html"
    )

    # When
    result = get_user_biz_reviews(page_no_reviews)

    # Then
    expected = []
    assert set(result) == set(expected)



from typing import TypedDict

from yelp_fetcher._util import fetch_html
from yelp_fetcher.scraper.biz import get_biz_review_url, review_id_on_page


class ReviewStatus(TypedDict):
    user_id: str
    biz_id: str
    review_id: str
    is_alive: str


def fetch_status(user_id, biz_id, review_id):
    url = get_biz_review_url(biz_id, review_id)
    html = fetch_html(url)
    is_alive = review_id_on_page(html, review_id)
    return ReviewStatus(
        user_id=user_id, biz_id=biz_id, review_id=review_id, is_alive=is_alive
    )



import re

from yelp_fetcher.scraper.bs4_util import (
    get_element_by_classname,
    get_elements_by_classname,
)


def get_review_ids(page):
    review_ids = map(
        lambda r: r["data-review-id"], get_elements_by_classname(page, "review")
    )
    return list(review_ids)


def get_num_pages(page):
    page_of_pages_element = get_element_by_classname(page, "page-of-pages")
    if page_of_pages_element is None:
        return 0
    matches = re.search(r"Page ([0-9]+) of ([0-9]+)", page_of_pages_element.get_text())
    curr_page, max_pages = matches.groups()
    return int(max_pages)



from typing import List, TypedDict

from yelp_fetcher._util import fetch_soup
from yelp_fetcher.scraper.common import get_num_pages
from yelp_fetcher.scraper.user_details_reviews_self import (
    ScrapedReview,
    get_user_biz_reviews,
    get_user_details_reviews_self_url,
)


class Review(TypedDict):
    user_id: str
    biz_id: str
    biz_name: str
    review_id: str
    review_date: str

    @staticmethod
    def from_scraped_review(user_id, scraped_review: ScrapedReview):
        return Review(
            user_id=user_id,
            biz_id=scraped_review.biz_id,
            biz_name=scraped_review.biz_name,
            review_id=scraped_review.review_id,
            review_date=scraped_review.review_date,
        )


def get_urls(user_id) -> List[str]:
    url = get_user_details_reviews_self_url(user_id)
    first_user_biz_soup = fetch_soup(url)
    num_user_biz_pages = get_num_pages(first_user_biz_soup)
    urls = map(
        lambda i: get_user_details_reviews_self_url(user_id, i),
        range(num_user_biz_pages),
    )
    return list(urls)


def fetch_reviews(user_id, url) -> List[Review]:
    user_biz_soup = fetch_soup(url)
    scraped_reviews = get_user_biz_reviews(user_biz_soup)
    reviews = map(lambda x: Review.from_scraped_review(user_id, x), scraped_reviews)
    return list(reviews)



from tests.scraper.resources import get_test_resource_soup
from yelp_fetcher.scraper.common import get_num_pages, get_review_ids


def test_get_review_ids_user_review_page():
    # Given
    soup_reviews_page = get_test_resource_soup(
        "user_details_reviews_self/user_details_reviews_self?userid=5prk8CtPPBHNpa6BOja2ug.html"
    )

    # When
    result = get_review_ids(soup_reviews_page)

    # Then
    expected = [
        "ofWhnV6m26kMyCpUjT56XA",
        "MS1f3LaFfHeh5kcF2qt-1A",
        "kzJihaxgEIphSySgBNNOiA",
        "7lJZPiK5-rBkojTxYS6R2w",
        "X0_-zEk0_CKj24wEEoau9A",
        "Ve3BLwnfMTXZMwCViW-MLg",
        "ZfHJEeZFKTPFwAJGMYiXrA",
        "dy_dxor2rVhpNuSUrrBw0w",
        "-eb-SRIXvQ_tyJKFldJMyw",
        "A_mJavYl2b4_oitezbpR8w",
    ]
    assert set(result) == set(expected)


def test_get_review_ids_not_recommended_page():
    # Given
    soup_not_recc = get_test_resource_soup(
        "not_recommended_reviews/not_recommended_reviews_súp-noodle-bar-buena-park-buena-park.html"
    )

    # When
    result = get_review_ids(soup_not_recc)

    # Then
    expected = [
        "04-z0KafkNno3SJROYOZXg",
        "6_Q_M3P9m-2RCKdJzvAWpw",
        "7mDLwDZDRB_8NOxv32F1ZQ",
        "94e2ewMKPQVZ1PmnS4uUSQ",
        "CTpZ1v4rGDM55jJ3S3yxZg",
        "ISo45SEdiOdLWZDVXBcMwQ",
        "J62o0SpdCIsANZx8seufEg",
        "O6aj9IDB4kwn7CY-L7cfAQ",
        "W17B30zLlM9zcy0LhH8AfA",
        "X8guNsPPamxl8568pwuHdw",
        "Xml9y-R6zuc1vL3jZXGAzA",
        "ZNz_v0117JiXIS4xu1xfRw",
        "eKE5nnpUmJ79QQ5OHiCTqA",
        "hRIocQbe87YebNkBgZfqCQ",
        "mHrxc6L0-E1Hu32hN-TSEg",
        "mfTknfdIohktWouPgqS2wQ",
        "nbSZnnEoUSv7IrwRdDnFMw",
        "rAjxjUm-x5SQ3XDii1-4AA",
        "rh7E1MRtAJ8Ql4JtAjwhEA",
        "riucV6WDfaDBzH5DUH3hOQ",
    ]
    assert set(result) == set(expected)


def test_get_review_ids_not_recommended_page_none():
    # Given
    soup_not_recc = get_test_resource_soup(
        "not_recommended_reviews/not_recommended_reviews_pho-ngon-norwalk-norwalk-2.html"
    )

    # When
    result = get_review_ids(soup_not_recc)

    # Then
    assert result == []


def test_get_num_pages_user_review_page():
    # Given
    soup_reviews_page = get_test_resource_soup(
        "user_details_reviews_self/user_details_reviews_self?userid=5prk8CtPPBHNpa6BOja2ug.html"
    )

    # When
    result = get_num_pages(soup_reviews_page)

    # Then
    assert result == 2


def test_get_num_pages_not_recommended_page():
    # Given
    soup_not_recc = get_test_resource_soup(
        "not_recommended_reviews/not_recommended_reviews_súp-noodle-bar-buena-park-buena-park.html"
    )

    # When
    result = get_num_pages(soup_not_recc)

    # Then
    assert result == 13


def test_get_num_pages_none():
    # Given
    soup_not_recc = get_test_resource_soup(
        "user_details_reviews_self/user_details_reviews_self?userid=yOdiJSKdFuE6EsAfd7jDKQ.html"
    )

    # When
    result = get_num_pages(soup_not_recc)

    # Then
    assert result == 0



import re

from yelp_fetcher.scraper.bs4_util import get_element_by_classname

USER_URL = "https://www.yelp.com/user_details?userid={}"


def get_user_details_url(user_id):
    return USER_URL.format(user_id)


def get_user_name(page):
    return get_element_by_classname(page, "user-profile_info").h1.text


def get_user_city(page):
    return get_element_by_classname(page, "user-location").text


def get_user_review_count(page):
    text = get_element_by_classname(page, "review-count").text
    return re.search(r"([0-9]+)", text).group()



import setuptools

setuptools.setup(
    name="yelp-common",
    version="1.1.3",
    author="Sam Kim",
    author_email="samkim90274@gmail.com",
    url="https://github.com/sampromises/yelp-common",
    packages=setuptools.find_packages(exclude=("tests",)),
    install_requires=["bs4", "requests", "dateparser",],
    python_requires=">=3.8",
)



from datetime import datetime
from typing import TypedDict

from yelp_fetcher._util import fetch_soup
from yelp_fetcher.scraper.user_details import (
    get_user_city,
    get_user_details_url,
    get_user_name,
    get_user_review_count,
)


class Metadata(TypedDict):
    user_id: str
    name: str
    city: str
    review_count: str
    last_updated: str


def fetch_metadata(user_id: str) -> Metadata:
    url = get_user_details_url(user_id)
    user_soup = fetch_soup(url)
    return Metadata(
        user_id=user_id,
        name=get_user_name(user_soup),
        city=get_user_city(user_soup),
        review_count=get_user_review_count(user_soup),
        last_updated=datetime.now().isoformat(),
    )



import re
from collections import namedtuple
from typing import List

from yelp_fetcher._util import fetch_soup
from yelp_fetcher.scraper.bs4_util import get_elements_by_classname
from yelp_fetcher.scraper.common import get_review_ids


class ScrapedReview(
    namedtuple("ScrapedReview", "biz_id biz_name review_id review_date")
):
    pass


USER_REVIEW_URL = (
    "https://www.yelp.com/user_details_reviews_self?rec_pagestart={}&userid={}"
)
DATE_REGEX = re.compile(r"[0-9]{1,2}/[0-9]{1,2}/[0-9]{4}")


def get_user_details_reviews_self_url(user_id, page=0):
    page_start = page * 10
    return USER_REVIEW_URL.format(page_start, user_id)


def get_user_biz_reviews(page) -> List[ScrapedReview]:
    biz_elems = get_elements_by_classname(page, "biz-name")
    biz_ids = list(map(lambda elem: elem["href"].split("/")[-1], biz_elems))
    biz_names = list(map(lambda elem: elem.get_text(), biz_elems))

    review_ids = get_review_ids(page)

    review_date_elems = get_elements_by_classname(page, "rating-qualifier")
    review_dates = list(
        map(lambda e: re.search(DATE_REGEX, e.get_text()).group(), review_date_elems)
    )

    tups = zip(biz_ids, biz_names, review_ids, review_dates)
    scraped_reviews = list(map(lambda tup: ScrapedReview(*tup), tups))
    return scraped_reviews


get_user_biz_reviews(
    fetch_soup(get_user_details_reviews_self_url("5prk8CtPPBHNpa6BOja2ug", 0))
)



import os

from bs4 import BeautifulSoup

THIS_DIR = os.path.dirname(os.path.abspath(__file__))


def get_test_resource_path(path):
    """Get file given a path, relative to scraper/tests/html"""
    result = os.path.join(THIS_DIR, "html", path)
    return result


def get_test_resource_html(path):
    """Get HTML given a path, relative to scraper/tests/html"""
    with open(get_test_resource_path(path)) as f:
        return f.read()


def get_test_resource_soup(path):
    """Get BS4 BeautifulSoup given a path, relative to scraper/tests/html"""
    with open(get_test_resource_path(path)) as f:
        return BeautifulSoup(f.read(), "html.parser")



from tests.scraper.resources import get_test_resource_path
from yelp_fetcher.scraper.bs4_util import (
    get_element_by_classname,
    get_elements_by_classname,
    to_soup,
)


def test_to_soup():
    # Given
    with open(get_test_resource_path("bs4_util/sam_kim_yelp.html")) as f:
        text = f.read()

    # When
    soup = to_soup(text)

    # Then
    assert soup
    assert soup.title.string == "Sam K.'s Reviews | Irvine - Yelp"


def test_get_elements_by_classname_none():
    # Given
    with open(get_test_resource_path("bs4_util/sam_kim_yelp.html")) as f:
        text = f.read()
        soup = to_soup(text)

    # When
    elements = get_elements_by_classname(soup, "this-class-doesnt-exist")

    # Then
    assert len(elements) == 0


def test_get_elements_by_classname_exists():
    # Given
    with open(get_test_resource_path("bs4_util/sam_kim_yelp.html")) as f:
        text = f.read()
        soup = to_soup(text)

    # When
    elements = get_elements_by_classname(soup, "titled-nav_item")

    # Then
    assert len(elements) == 5


def test_get_element_by_classname_none():
    # Given
    with open(get_test_resource_path("bs4_util/sam_kim_yelp.html")) as f:
        text = f.read()
        soup = to_soup(text)

    # When
    result = get_element_by_classname(soup, "this-class-doesnt-exist")

    # Then
    assert result is None


def test_get_element_by_classname_exists():
    # Given
    with open(get_test_resource_path("bs4_util/sam_kim_yelp.html")) as f:
        text = f.read()
        soup = to_soup(text)

    # When
    element = get_element_by_classname(soup, "user-profile_info")

    # Then
    assert element
